package com.sun.xml.internal.ws.util;

import java.util.ArrayList;
import java.util.List;
import com.sun.istack.internal.NotNull;
import com.sun.istack.internal.Nullable;
import java.util.Iterator;
import javax.xml.namespace.NamespaceContext;
import org.w3c.dom.NamedNodeMap;
import javax.xml.stream.XMLStreamException;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import javax.xml.stream.XMLStreamWriter;
import org.w3c.dom.Element;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.FactoryConfigurationError;
import com.sun.xml.internal.ws.util.xml.XmlUtil;
import org.w3c.dom.Document;
import javax.xml.parsers.DocumentBuilder;

public class DOMUtil
{
    private static DocumentBuilder db;
    
    public DOMUtil() {
    }
    
    public static Document createDom() {
        synchronized (DOMUtil.class) {
            if (DOMUtil.db == null) {
                try {
                    DocumentBuilderFactory dbf = XmlUtil.newDocumentBuilderFactory();
                    DOMUtil.db = dbf.newDocumentBuilder();
                }
                catch (ParserConfigurationException e) {
                    throw new FactoryConfigurationError(e);
                }
            }
            return DOMUtil.db.newDocument();
        }
    }
    
    public static void serializeNode(Element node, XMLStreamWriter writer) throws XMLStreamException {
        writeTagWithAttributes(node, writer);
        if (node.hasChildNodes()) {
            NodeList children = node.getChildNodes();
            for (int i = 0; i < children.getLength(); ++i) {
                Node child = children.item(i);
                switch (child.getNodeType()) {
                    case 7: {
                        writer.writeProcessingInstruction(child.getNodeValue());
                        break;
                    }
                    case 10: {
                        break;
                    }
                    case 4: {
                        writer.writeCData(child.getNodeValue());
                        break;
                    }
                    case 8: {
                        writer.writeComment(child.getNodeValue());
                        break;
                    }
                    case 3: {
                        writer.writeCharacters(child.getNodeValue());
                        break;
                    }
                    case 1: {
                        serializeNode((Element)child, writer);
                        break;
                    }
                }
            }
        }
        writer.writeEndElement();
    }
    
    public static void writeTagWithAttributes(Element node, XMLStreamWriter writer) throws XMLStreamException {
        String nodePrefix = fixNull(node.getPrefix());
        String nodeNS = fixNull(node.getNamespaceURI());
        String nodeLocalName = (node.getLocalName() == null) ? node.getNodeName() : node.getLocalName();
        boolean prefixDecl = isPrefixDeclared(writer, nodeNS, nodePrefix);
        writer.writeStartElement(nodePrefix, nodeLocalName, nodeNS);
        if (node.hasAttributes()) {
            NamedNodeMap attrs = node.getAttributes();
            for (int numOfAttributes = attrs.getLength(), i = 0; i < numOfAttributes; ++i) {
                Node attr = attrs.item(i);
                String nsUri = fixNull(attr.getNamespaceURI());
                if (nsUri.equals("http://www.w3.org/2000/xmlns/")) {
                    String local = attr.getLocalName().equals("xmlns") ? "" : attr.getLocalName();
                    if (local.equals(nodePrefix) && attr.getNodeValue().equals(nodeNS)) {
                        prefixDecl = true;
                    }
                    if (local.equals("")) {
                        writer.writeDefaultNamespace(attr.getNodeValue());
                    }
                    else {
                        writer.setPrefix(attr.getLocalName(), attr.getNodeValue());
                        writer.writeNamespace(attr.getLocalName(), attr.getNodeValue());
                    }
                }
            }
        }
        if (!prefixDecl) {
            writer.writeNamespace(nodePrefix, nodeNS);
        }
        if (node.hasAttributes()) {
            NamedNodeMap attrs = node.getAttributes();
            for (int numOfAttributes = attrs.getLength(), i = 0; i < numOfAttributes; ++i) {
                Node attr = attrs.item(i);
                String attrPrefix = fixNull(attr.getPrefix());
                String attrNS = fixNull(attr.getNamespaceURI());
                if (!attrNS.equals("http://www.w3.org/2000/xmlns/")) {
                    String localName = attr.getLocalName();
                    if (localName == null) {
                        localName = attr.getNodeName();
                    }
                    boolean attrPrefixDecl = isPrefixDeclared(writer, attrNS, attrPrefix);
                    if (!attrPrefix.equals("") && !attrPrefixDecl) {
                        writer.setPrefix(attr.getLocalName(), attr.getNodeValue());
                        writer.writeNamespace(attrPrefix, attrNS);
                    }
                    writer.writeAttribute(attrPrefix, attrNS, localName, attr.getNodeValue());
                }
            }
        }
    }
    
    private static boolean isPrefixDeclared(XMLStreamWriter writer, String nsUri, String prefix) {
        boolean prefixDecl = false;
        NamespaceContext nscontext = writer.getNamespaceContext();
        Iterator<String> prefixItr = nscontext.getPrefixes(nsUri);
        while (prefixItr.hasNext()) {
            if (!prefix.equals(prefixItr.next())) {
                continue;
            }
            prefixDecl = true;
            break;
        }
        return prefixDecl;
    }
    
    public static Element getFirstChild(Element e, String nsUri, String local) {
        for (Node n = e.getFirstChild(); n != null; n = n.getNextSibling()) {
            if (n.getNodeType() == 1) {
                Element c = (Element)n;
                if (c.getLocalName().equals(local) && c.getNamespaceURI().equals(nsUri)) {
                    return c;
                }
            }
        }
        return null;
    }
    
    @NotNull
    private static String fixNull(@Nullable String s) {
        if (s == null) {
            return "";
        }
        return s;
    }
    
    @Nullable
    public static Element getFirstElementChild(Node parent) {
        Node n = parent.getFirstChild();
        while (n != null) {
            if (n.getNodeType() == 1) {
                return (Element)n;
            }
            n = n.getNextSibling();
        }
        return null;
    }
    
    @NotNull
    public static List<Element> getChildElements(Node parent) {
        List<Element> elements = new ArrayList();
        for (Node n = parent.getFirstChild(); n != null; n = n.getNextSibling()) {
            if (n.getNodeType() == 1) {
                elements.add((Element)n);
            }
        }
        return elements;
    }
}
